/*========================================================================
 * $RCSfile: PatternEditorModel.java,v $
 *
 * $Author: liddle $
 * $Revision: 1.1 $
 * $Date: 1998/08/06 16:29:07 $
 *
 * $Log: PatternEditorModel.java,v $
 * Revision 1.1  1998/08/06  16:29:07  liddle
 * Initial revision
 *
 *------------------------------------------------------------------------
 * Pattern Editor
 * Data Extraction Research Group
 * School of Accountancy and Information Systems
 * Marriott School of Management
 * 540 TNRB
 * Brigham Young University
 * Provo, UT 84602-3068
 * liddle@byu.edu
 *
 * Copyright (C) 1998, Board of Trustees of Brigham Young University
 *
 * Pattern Editor software, both binary and source (hereafter, Software)
 * is copyrighted by The Board of Trustees of Brigham Young University
 * (BYU), and ownership remains with BYU.
 *
 * BYU grants you (hereafter, Licensee) a license to use the Software
 * for academic, research and internal business purposes only, without a
 * fee.  Licensee may distribute the binary and source code (if released)
 * to third parties provided that the copyright notice and this statement
 * appears on all copies and that no charge is associated with such
 * copies.
 *
 * Licensee may not make derivative works without first arranging an
 * additional license agreement with BYU.
 *
 * Any Licensee wishing to make commercial use of the Software should
 * contact BYU to negotiate an appropriate license for such
 * commercial use.  Commercial use includes (1) integration of all or
 * part of the source code into a product for sale or license by or on
 * behalf of Licensee to third parties, or (2) distribution of the binary
 * code or source code to third parties that need it to utilize a
 * commercial product sold or licensed by or on behalf of Licensee.
 *
 * BYU MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.  BYU SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE
 * USERS OF THIS SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the United States
 * including, but not limited to, export control laws, and the terms
 * of this license.  BYU shall have the right to terminate this license
 * immediately by written notice upon Licensee's breach of, or
 * non-compliance with, any of its terms.  Licensee may be held legally
 * responsible for any copyright infringement that is caused or
 * encouraged by Licensee's failure to abide by the terms of this license.
 *
 * Comments and questions are welcome and can be sent to liddle@byu.edu.
 */
/*------------------------------------------------------------------------
 *			IMPORTS
 */
import java.awt.*;
import java.io.*;

import com.sun.java.swing.text.*;
import com.sun.java.swing.event.*;

/*========================================================================
 *			CLASS PatternEditorModel
 */
public class PatternEditorModel {

/*------------------------------------------------------------------------
 *			PUBLIC CONSTANTS
 */
public static final int		CASE_COL_NUM = 0,
				ID_COL_NUM = 1,
				PATTERN_COL_NUM = 2,
				NUM_COLUMNS = 3;
public static final String	UNTITLED_NAME = "Untitled";

/*------------------------------------------------------------------------
 *			ATTRIBUTES
 */
private PatternTableModel	pattern_model;
private String			pattern_fname = "patterns.txt",
				pattern_path,
				report_fname = "rp0010",
				report_path;
private boolean			modified = false;
private Report			report;
private StyledDocument		report_doc;
private Style			dflt_style;

private com.oroinc.text.regex.Perl5Matcher
		matcher = new com.oroinc.text.regex.Perl5Matcher();

/*------------------------------------------------------------------------
 *			CONSTRUCTORS
 */
public
PatternEditorModel(String pfile, String rfile, PatternEditor ped)
	throws IOException
{
  GapContent	gapc;
  StyleContext	sc;
  String	cur_dir;

  cur_dir = new File(".").getAbsolutePath();
  cur_dir = cur_dir.substring(0,
	cur_dir.lastIndexOf(File.separatorChar) + 1).replace('\\', '/');

  pattern_fname = pfile.replace('\\', '/');
  report_fname = rfile.replace('\\', '/');

  if (Is_Absolute_Path(pattern_fname)) {
    // Absolute path specification.
    pattern_path = pattern_fname;
  } else {
    // Relative path specification.
    pattern_path = cur_dir + pattern_fname;
  }
  // Separate directory path from file name.
  pattern_fname = pattern_path.substring(
  				      pattern_path.lastIndexOf('/') + 1);
  pattern_path = pattern_path.substring(0,
  				      pattern_path.lastIndexOf('/') + 1);

  if (Is_Absolute_Path(report_fname)) {
    // Absolute path specification.
    report_path = report_fname;
  } else {
    // Relative path specification.
    report_path = cur_dir + report_fname;
  }
  // Separate directory path from file name.
  report_fname = report_path.substring(report_path.lastIndexOf('/') + 1);
  report_path = report_path.substring(0,
  				      report_path.lastIndexOf('/') + 1);

  pattern_model = new PatternTableModel();

  gapc = new GapContent(32768);
  sc = new StyleContext();
  dflt_style = sc.addStyle(null, null);
  StyleConstants.setForeground(dflt_style, Color.black);
  report_doc = new DefaultStyledDocument(gapc, sc);

  Load_Report();
  if (Load_Patterns())
    ped.Set_StatusBar("Unable to load patterns file.");

} /* PatternEditorModel constructor */

/*------------------------------------------------------------------------
 *			PUBLIC METHODS
 */
public void
Add_Pattern()
{
  pattern_model.Add_Pattern();

} /* Add_Pattern */

public void
Add_PatternTableModel_Listener(TableModelListener plist_update)
{
  pattern_model.addTableModelListener(plist_update);

} /* Add_PatternTableModel_Listener */

public PatternVector
Clone_Patterns(int [] sel_rows, int sel_cnt)
{
  PatternVector	to_copy = new PatternVector();
  PatternVector	patterns = pattern_model.Get_Patterns();

  for (int i = 0; i < sel_cnt; i++)
    to_copy.addElement(patterns.Get(sel_rows[i]).clone());
  return (to_copy);

} /* Clone_Patterns */

public StyledDocument
Get_Document()
{
  return (report_doc);

} /* Get_Document */

public int
Get_Num_Patterns()
{
  return (pattern_model.Get_Patterns().size());

} /* Get_Num_Patterns */

public String
Get_Pattern(int ix)
{
  return (pattern_model.Get_Patterns().Get(ix).toString());

} /* Get_Pattern */

public PatternVector.Pattern
Get_Pattern_At(int ix)
{
  return (pattern_model.Get_Patterns().Get(ix));

} /* Get_Pattern */

public String
Get_Pattern_Filename()
{
  return (pattern_fname);

} /* Get_Pattern_Filename */

public String
Get_Pattern_Path()
{
  return (pattern_path);

} /* Get_Pattern_Path */

public PatternTableModel
Get_PatternTableModel()
{
  return (pattern_model);

} /* Get_Pattern_Table_Model */

public String
Get_Report_Filename()
{
  return (report_fname);

} /* Get_Report_Filename */

public String
Get_Report_Path()
{
  return (report_path);

} /* Get_Report_Path */

public String
Get_Report_Text()
{
  return (report.toString());

} /* Get_Report_Text */

public void
Insert_Patterns(PatternVector pats, int row)
{
  pattern_model.Insert_Patterns(pats, row);

} /* Insert_Patterns */

public boolean
Is_Absolute_Path(String path)
{
  if (path.charAt(0) == '/' || path.charAt(0) == '\\')
    return (true);
  if (path.charAt(2) == '/' || path.charAt(2) == '\\')
    if (path.charAt(1) == ':')
      return (Character.isLetter(path.charAt(0)));
  return (false);

} /* Is_Absolute_Path */

public boolean
Is_Modified()
{
  return (modified);

} /* Is_Modified */

public boolean
Is_PatternVector_Untitled()
{
  return (pattern_fname.equals(UNTITLED_NAME));

} /* Is_PatternVector_Untitled */

public boolean
Load_Patterns()
{
  modified = false;
  if (pattern_model.Load_Patterns(pattern_path + pattern_fname)) {
    pattern_fname = UNTITLED_NAME;
    return (true);
  }
  return (false);

} /* Load_Patterns */

public void
Load_Report()
{
  report = new Report(report_path + report_fname);
  try {
    if (report_doc.getLength() > 0)
      report_doc.remove(0, report_doc.getLength());
    report_doc.insertString(0, Get_Report_Text(), dflt_style);
  } catch (BadLocationException e) {
    // status_bar.setText("Bad Location Exception: " + e);
  }

} /* Load_Report */

public void
Remove_Patterns(int [] sel_rows, int sel_cnt)
{
  pattern_model.Remove_Patterns(sel_rows, sel_cnt);

} /* Remove_Patterns */

public int
Reorder_Patterns( boolean move_up, int sel_cnt,
		  int [] sel_rows, int delta )
{
  return ( pattern_model.Reorder_Patterns(move_up, sel_cnt,
  					  sel_rows, delta) );
} /* Reorder_Patterns */

public void
Set_Case_Sensitivity_At(int row, boolean value)
{
  pattern_model.setValueAt(new Boolean(value), row, CASE_COL_NUM);

} /* Set_Case_Sensitivity_At */

public void
Set_Modified(boolean value)
{
  modified = value;

} /* Set_Modified */

public void
Set_Pattern_Filename(String new_file)
{
  pattern_fname = new_file;

} /* Set_Pattern_Filename */

public void
Set_Pattern_Path(String new_path)
{
  pattern_path = new_path;

} /* Set_Pattern_Path */

public void
Set_Report_Filename(String new_file)
{
  report_fname = new_file;

} /* Set_Report_Filename */

public void
Set_Report_Path(String new_path)
{
  report_path = new_path;

} /* Set_Report_Path */

public void
Write_Patterns() throws IOException
{
  // Write patterns out to given file.
  PrintWriter	pout = new PrintWriter(
			new FileWriter(pattern_path + pattern_fname));
  PatternVector	patterns = pattern_model.Get_Patterns();

  for (int i = 0; i < patterns.size() - 1; i++)
    pout.println(patterns.Get(i).toString());
  pout.close();
  modified = false;

} /* Write_File */

} /* class PatternEditorModel */

/*========================================================================
 *			END OF FILE $RCSfile: PatternEditorModel.java,v $
 */
